package benchmark.geomesa.insert;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFinder;
import org.geotools.data.FeatureStore;
import org.geotools.data.simple.SimpleFeatureStore;
import org.geotools.feature.DefaultFeatureCollection;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.SchemaException;
import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.locationtech.geomesa.utils.interop.SimpleFeatureTypes;
import org.locationtech.geomesa.utils.interop.WKTUtils;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;

import com.beust.jcommander.internal.Lists;
import com.google.common.base.Joiner;
import com.vividsolutions.jts.geom.Geometry;

import benchmark.base.database.TSDBInsert;

public abstract class GeomesaInsert extends TSDBInsert{
	
    protected DataStore dataStore;
    protected String simpleFeatureTypeName;
    protected SimpleFeatureType simpleFeatureType;

	public GeomesaInsert() throws Exception{
		Map<String, String> dsConf = getDatastoreConf();
		dataStore = DataStoreFinder.getDataStore(dsConf);
		assert dataStore != null;
		simpleFeatureTypeName = "Pointn";
		simpleFeatureType = createSimpleFeatureType(simpleFeatureTypeName);
		dataStore.createSchema(simpleFeatureType);
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void commit() {
		FeatureCollection featureCollection = createNewFeatures(simpleFeatureType);
		try {
			insertFeatures(simpleFeatureTypeName, dataStore, featureCollection);
			points.clear();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	
	@Override
	public void end(){
		super.end();
		dataStore.dispose();
	}
	
	public abstract Map<String, String> getDatastoreConf();
	
	static SimpleFeatureType createSimpleFeatureType(String simpleFeatureTypeName)
            throws SchemaException {
		
        // list the attributes that constitute the feature type
        List<String> attributes = Lists.newArrayList(
        		"PointId:java.lang.Integer:index=full",
        		"Name:String",
        		"*Position:Point:srid=4326",
        		"Date:Date",
        );

        // create the bare simple-feature type
        String simpleFeatureTypeSchema = Joiner.on(",").join(attributes);
        SimpleFeatureType simpleFeatureType =
                SimpleFeatureTypes.createType(simpleFeatureTypeName, simpleFeatureTypeSchema);

        // use the user-data (hints) to specify which date-time field is meant to be indexed;
        // if you skip this step, your data will still be stored, it simply won't be indexed
        simpleFeatureType.getUserData().put(SimpleFeatureTypes.DEFAULT_DATE_KEY, "Date");

        return simpleFeatureType;
    }
	
	@SuppressWarnings("rawtypes")
	protected FeatureCollection createNewFeatures(SimpleFeatureType simpleFeatureType) {
        DefaultFeatureCollection featureCollection = new DefaultFeatureCollection();
        
        Object[] NO_VALUES = {};

        for (int i = 0; i < points.size() ; i ++) {
            SimpleFeature simpleFeature = SimpleFeatureBuilder.build(simpleFeatureType, NO_VALUES, "point."+i);

            // be sure to tell GeoTools explicitly that you want to use the ID you provided
            //simpleFeature.getUserData().put(Hints.USE_PROVIDED_FID, java.lang.Boolean.TRUE);

            // populate the new feature's attributes

            simpleFeature.setAttribute("PointId", points.get(i).getId());
            simpleFeature.setAttribute("Name", points.get(i).getName());
            
            Geometry geo = WKTUtils.read("POINT("+points.get(i).getLongitude() + " " + points.get(i).getLatitude() +")");
            simpleFeature.setAttribute("Position", geo);
            
            Date date = new Date(points.get(i).getTimestamp());
            simpleFeature.setAttribute("Date", date);
            
            // accumulate this new feature in the collection
            featureCollection.add(simpleFeature);
        }

        return featureCollection;
    }
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected void insertFeatures(String simpleFeatureTypeName, DataStore dataStore, FeatureCollection featureCollection)
	throws IOException {
	
		FeatureStore<?, ?> featureStore = (SimpleFeatureStore) dataStore.getFeatureSource(simpleFeatureTypeName);
		featureStore.addFeatures(featureCollection);
	}
	
	public String getSimpleFeatureTypeName(){
		return simpleFeatureTypeName;
	}

}
