package benchmark.base.database;

import java.util.ArrayList;
import java.util.Date;

import benchmark.base.model.Point;

/**
 * Insertion into database
 * @author rgrisot
 *
 */
public abstract class TSDBInsert implements Runnable{

	protected ArrayList<Point> points;
	protected int bucketSize; 

	/**
	 * Define size of packs to send to DB
	 * @param bucketSize : number of datas to send at once to the DB
	 */
	public TSDBInsert(int bucketSize){
		points = new ArrayList<Point>();
		this.bucketSize = bucketSize;
	}
	
	/**
	 * Default value : 100 000
	 */
	public TSDBInsert() {
		this(100_000);
	}
	
	/**
	 * Send datas to DB
	 */
	public abstract void commit();
	
	/**
	 * Add poitn to the list to send
	 * @param point : point to insert
	 */
	public void insert(Point p){
		points.add(p);
		if(points.size() >= bucketSize){
			commit();
		}
	}
	
	
	/**
	 * end connection to DB (to define in subclasses) and send last datas to DB
	 */
	public void end(){
		if(points.size() > 0){
			commit();
		}
	}
	
	/***
	 * Set list of points
	 * @param points : list of points
	 */
	public void setPoints(ArrayList<Points> points){
		this.points = points;
	}
	
	/**
	 * Datas are inserted while other are collected in same time
	 */
	@Override
	public void run() {
		commit();
		end();
	}
	
	/**
	 * 
	 * @return db name
	 */
	public abstract String getName();

}
